package cz.jzitnik;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.SpriteLoader;
import cz.jzitnik.tui.ScreenRenderer;
import org.jline.terminal.Terminal;
import org.jline.terminal.TerminalBuilder;

import java.io.IOException;

public class Main {
    public static void main(String[] args) {
        try {
            // Set up terminal with JLine
            Terminal terminal = TerminalBuilder.terminal();
            terminal.enterRawMode(); // Switch to raw mode to capture keys immediately
            var spriteList = SpriteLoader.load();
            var screenRenderer = new ScreenRenderer(spriteList);
            var game = new Game();

            final boolean[] isRunning = {true};

            Thread inputThread = new Thread(() -> {
                try {
                    while (isRunning[0]) {
                        int key = terminal.reader().read();
                        switch (key) {
                            case 'a':
                                game.movePlayerLeft(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case 'd':
                                game.movePlayerRight(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case ' ':
                                game.movePlayerUp(screenRenderer);
                                screenRenderer.render(game.getWorld());
                                break;
                            case 'm':
                                System.out.println("Mine pressed");
                                break;
                            case 'q':
                                System.out.println("Exiting game...");
                                isRunning[0] = false;
                                break;
                            default:
                                break;
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            });

            inputThread.start();

            // Game loop (rendering the game)
            while (isRunning[0]) {
                screenRenderer.render(game.getWorld());
                try {
                    Thread.sleep(1000); // Control game loop speed
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            terminal.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
