package cz.jzitnik.game;

import cz.jzitnik.tui.ScreenRenderer;
import lombok.Getter;
import java.util.ArrayList;
import java.util.List;

@Getter
public class Game {
    private List<Block>[][] world = new ArrayList[50][50];
    private Block player;

    public Game() {
        for (int i = 0; i < 50; i++) {
            for (int j = 0; j < 50; j++) {
                world[i][j] = new ArrayList<>();
            }
        }

        Block steveBlock = new Block("steve", SpriteLoader.SPRITES.STEVE);
        world[10][2].add(new Block("grass", SpriteLoader.SPRITES.GRASS));
        player = steveBlock;

        for (int i = 0; i < 50; i++) {
            world[11][i].add(new Block("grass", SpriteLoader.SPRITES.GRASS));
            world[12][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));
            world[13][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));
            world[14][i].add(new Block("dirt", SpriteLoader.SPRITES.DIRT));

            for (int j = 15; j < 49; j++) {
                world[j][i].add(new Block("stone", SpriteLoader.SPRITES.STONE));
            }

            world[49][i].add(new Block("bedrock", SpriteLoader.SPRITES.BEDROCK));
        }

        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                if (world[i][j].isEmpty()) {
                    world[i][j].add(new Block("air", SpriteLoader.SPRITES.AIR, true)); // Fill with air
                }
            }
        }

        world[10][0].add(steveBlock);
    }

    private int[] getPlayerCords() {
        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                for (Block block : world[i][j]) {
                    if (block.getBlockId().equals("steve")) {
                        return new int[]{j, i};
                    }
                }
            }
        }
        return null;
    }

    public void movePlayerRight(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1]][cords[0] + 1].stream().anyMatch(block -> !block.isGhost())) {
            return;
        }

        world[cords[1]][cords[0] + 1].add(player);
        world[cords[1]][cords[0]].remove(player);


        new Thread(() -> {
            while (true) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                int[] cords2 = getPlayerCords();

                if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                    world[cords2[1] + 1][cords2[0]].add(player);
                    world[cords2[1]][cords2[0]].remove(player);

                    screenRenderer.render(world);
                } else {
                    break;
                }
            }
        }).start();
    }

    public void movePlayerLeft(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1]][cords[0] - 1].stream().anyMatch(block -> !block.isGhost())) {
            return;
        }

        world[cords[1]][cords[0] - 1].add(player);
        world[cords[1]][cords[0]].remove(player);

        new Thread(() -> {
            while (true) {
                try {
                    Thread.sleep(200);
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                int[] cords2 = getPlayerCords();

                if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                    world[cords2[1] + 1][cords2[0]].add(player);
                    world[cords2[1]][cords2[0]].remove(player);

                    screenRenderer.render(world);
                } else {
                    break;
                }
            }
        }).start();
    }

    public void movePlayerUp(ScreenRenderer screenRenderer) {
        int[] cords = getPlayerCords();

        if (world[cords[1] - 1][cords[0]].stream().anyMatch(block -> !block.isGhost()) || world[cords[1] + 1][cords[0]].stream().anyMatch(Block::isGhost)) {
            return;
        }

        world[cords[1] - 1][cords[0]].add(player);
        world[cords[1]][cords[0]].remove(player);

        new Thread(() -> {
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            int[] cords2 = getPlayerCords();

            if (world[cords2[1] + 1][cords2[0]].stream().anyMatch(Block::isGhost)) {
                world[cords2[1] + 1][cords2[0]].add(player);
                world[cords2[1]][cords2[0]].remove(player);

                screenRenderer.render(world);
            }
        }).start();
    }
}
