package cz.jzitnik.tui;

import cz.jzitnik.game.Block;
import lombok.AllArgsConstructor;

import java.util.List;

@AllArgsConstructor
public class ScreenRenderer {
    private SpriteList spriteList;

    private int[] getPlayerCords(List<Block>[][] world) {
        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                if (world[i][j].stream().anyMatch(x -> x.getBlockId().equals("steve"))) {
                    return new int[]{j, i};
                }
            }
        }

        return null;
    }

    public void render(List<Block>[][] world) {
        StringBuilder main = new StringBuilder();
        main.append("\033[H\033[2J");

        int[] cords = getPlayerCords(world);
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];
        int viewXRadius = 5;
        int viewUpRadius = 4;
        int viewDownRadius = 3;

        // Calculate visible area boundaries
        int startX = Math.max(0, playerX - viewXRadius);
        int endX = Math.min(world[0].length, playerX + viewXRadius + 1);
        int startY = Math.max(0, playerY - viewUpRadius);
        int endY = Math.min(world.length, playerY + viewDownRadius + 1);

        StringBuilder[] lines = new StringBuilder[(endY - startY) * 25 + 1];
        for (int i = 0; i < lines.length; i++) {
            lines[i] = new StringBuilder();
        }

        int counter = 0;
        for (int y = startY; y < endY; y++) {
            for (int x = startX; x < endX; x++) {
                List<Block> blocks = world[y][x];
                String sprite = SpriteCombiner.combineSprites(blocks.stream()
                        .map(block -> spriteList.getSprite(block.getSprite()).getSprite())
                        .toArray(String[]::new));

                String[] spriteLines = sprite.split("\n");

                for (int i = 0; i < spriteLines.length; i++) {
                    lines[counter * 25 + i].append(spriteLines[i]);
                }
            }
            counter++;
        }

        for (StringBuilder line : lines) {
            main.append(line.toString());
            main.append("\n");
        }

        System.out.println(main);
    }
}
